      subroutine calc_rho(m,e,alt,pres,capa,grav,rho)
      implicit none
      include '../includes/max.inc'

      integer i,i1,i2,m
      double precision z1,z2,p1,p2
      double precision g1,g2,g
      double precision Cp1,Cp2,Cp
      double precision rho(1:Nmax)
      double precision e(0:Nmax),alt(0:Nmax)
      double precision pres(0:Nmax),capa(0:Nmax)
      double precision grav(0:Nmax)

c     dQe(i) reprsente maintenant la quantit d'nergie ncessaire  lever de 1K
c     la temprature de la maille i (dans le maillage de calcul "e").

      do i=1,m
c     Calcul des niveaux de pression p1 et p2, respectivement en bas et en haut
c     de la maille numro "i" du tableau "e" (p1>p2)
         z1=e(i-1)
         z2=e(i)
         call find_z_i1(m,alt,z1,i1)
         call find_z_i1(m,alt,z2,i2)
         call find_p(alt(i1),alt(i1+1),z1,pres(i1),pres(i1+1),p1)
         call find_p(alt(i2),alt(i2+1),z2,pres(i2),pres(i2+1),p2)
         call lin_interp(alt(i1),alt(i1+1),z1,capa(i1),capa(i1+1),Cp1)
         call lin_interp(alt(i2),alt(i2+1),z2,capa(i2),capa(i2+1),Cp2)
         Cp=(Cp1+Cp2)/2.0D+0
         call lin_interp(alt(i1),alt(i1+1),z1,grav(i1),grav(i1+1),g1)
         call lin_interp(alt(i2),alt(i2+1),z2,grav(i2),grav(i2+1),g2)
         g=(g1+g2)/2.0D+0
         rho(i)=(p1-p2)/(g*(e(i)-e(i-1))) ! kg/m
      enddo ! i

      return
      end
      

      subroutine find_z_i1(m,alt,z,i1)
      implicit none
      include '../includes/max.inc'

      integer i,i1,f,m
      double precision z,alt(0:Nmax)

c     Calcul de i1, le numro de la maille du tableau "alt" o
c     se situe l'altitude "z"
c     "z" sera donc situe entre les altitudes "alt(i1)" et "alt(i1+1)"

      f=0
      i=0
      do while (f.eq.0)
         if ((z.ge.alt(i)).and.(z.le.alt(i+1)))
     &        then
            f=1
            i1=i
         endif
         i=i+1
         if (i.gt.m-1) then
            write(*,*) 'Error in "find_z_i1":'
            write(*,*) 'Altitude ',z,' could not be located'
            stop
         endif
      enddo


      return
      end







      subroutine find_p(z1,z2,z,p1,p2,p)
      implicit none

      double precision p,p1,p2,z,z1,z2,a
c     Calcule la pression "p" correspondant  l'altitude "z",
c     sachant que 'p' est siut entre p1 (niveau de pression de bas de la
c     maille, situ  l'altitude z1) et p2 (niveau de pression de haut
c     de la maille, situ  l'altitude z2) en supposant une loi
c     d'volution en exponentielle de la pression entre les niveaux
c     z1 et z2: p(z)=p1*exp(-a*(z-z1))
      a=1.0D+0/(z2-z1)*dlog(p1/p2)
      p=p1*dexp(-a*(z-z1))

      return
      end







      subroutine lin_interp(z1,z2,z,a1,a2,a)
      implicit none

      double precision z1,z2,z,a1,a2,a

c     Interpolation linaire de la quantit "a":
c     a1 est connu en z1, a2 est connu en z2
c     on cherche a en z, en supposant une loi d'volution linaire
c     entre z1 et z2

      a=a1+(a2-a1)/(z2-z1)*(z-z1)

      return
      end
