c     Copyright (C) 2000-2008
c     Universite Paul Sabatier, Toulouse, France

c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c     
c     sensitivities...
      subroutine analytic_dNERdx(band,quad,dkadx,nspecies,
     &     i_min,i_max,ispecies,dner_dx)
      implicit none
      
      include 'max.inc'
      include 'data.inc'
      include 'options.inc'
      include 'param.inc'

      integer i,j,p,band,quad,md,it,jt,out
      integer nspecies,i_min,i_max,ispecies
      double precision z1,z2,z3,z4,kai,kaj
      double precision B1,B2,B3,B4,tau_B
      double precision B_sol,B_esp
      double precision Planck,expint,lambda
      double precision tau_atmos
      double precision dkadx(1:Nbmx,1:Nqmx,1:Nmax,1:Nsmx)
c     dner_dx(i,j,p) est la drive de la PNE(i,j) p.r.  la concentration x(p)
      double precision dner_dx(0:Nmax+1,0:Nmax+1,0:Nmax+1)
      double precision dnerd_dx(0:2*(Nmax+1),0:2*(Nmax+1),0:Nmax+1)
      double precision taui,tauj,tau_inf
      double precision func_dtaui_dxp
      double precision func_dtauB_dxp

      double precision dtau_inf_dxp
      double precision dtaui_dxp
      double precision dtauj_dxp
      double precision dtauB_dxp
      double precision dnerdij_dx_direct
      double precision alpha,beta
      double precision dnerdmij_dx_direct

      double precision B1_ind,B2_ind,B3_ind,B4_ind
      double precision taui_ind,tauj_ind,tau_B_ind
      double precision dtaui_dxp_ind
      double precision dtauj_dxp_ind
      double precision dtauB_dxp_ind
      double precision alpha_ind,beta_ind
      double precision dnerdij_dx_indirect

      double precision dNERij_dxp_dir,dNERij_dxp_ref
      double precision dNERis_dx_dir,dNERis_dx_ref
      double precision tauH,dtauH_dxp,tau_below,dtau_below_dxp


c     Computation of Planck intensities at ground and space levels
      lambda=(lambda_lo(band)+lambda_hi(band))/2.0D+0
      if (udif.eq.1) then
         B_sol=Planck(Tsol,lambda_lo(band),lambda_hi(band))
         B_esp=Planck(Tesp,lambda_lo(band),lambda_hi(band))
      else
         B_sol=udif_sol(band)
         B_esp=udif_esp(band)
      endif
      md=2*(m-1)

      do p=1,i_min-1
         do i=1,md
            dnerd_dx(0,i,p)=0.0D+0
            dnerd_dx(i,md+1,p)=0.0D+0
            do j=i+1,md
               dnerd_dx(i,j,p)=0.0D+0
            enddo ! j
         enddo ! i
         dnerd_dx(0,md+1,p)=0.0D+0
      enddo ! p

      do p=i_min,i_max
c     d(NER)/d(x) computation
         do i=1,md
            it=i/2+1
            z1=alti(i-1)
            z2=alti(i)
            call find_B(band,quad,z1,B1)
            call find_B(band,quad,z2,B2)
            call find_ka(band,quad,it,(z2+z1)/2.0D+0,kai)
            taui=kai*(z2-z1)
            dtaui_dxp=func_dtaui_dxp(i,p,dkadx,band,quad,ispecies)
c     Computation of NER(ground-layer i)
            call calc_tau_abs_mlay(band,quad,1,i-1,1,tau_B)
            tau_inf=tau_B
            dtauB_dxp=func_dtauB_dxp(0,i,p,dkadx,band,quad,ispecies)
            dtau_inf_dxp=dtauB_dxp
            
c     dner(0,i)/dx(p)
            if ((p.ge.0).and.(p.le.it+1)) then
               dnerd_dx(0,i,p)=-2.0D+0*pi*em_sol(band)*(
     &              -dtauB_dxp*(B_sol-B1)*expint(2,tau_B)+
     &              (dtauB_dxp+dtaui_dxp)*(B_sol-B2)
     &              *expint(2,tau_B+taui)+
     &              (B1-B2)/taui**2*(
     &              taui*(-dtauB_dxp*expint(3,tau_B)+
     &              (dtauB_dxp+dtaui_dxp)*expint(3,tau_B+taui))
     &              -dtaui_dxp*(expint(4,tau_B)-
     &              expint(4,tau_B+taui))))
            else if ((p.ge.it+2).and.(p.le.m+1)) then
               dnerd_dx(0,i,p)=0.0D+0
            endif
            
c     Computation of NER(space-layer i)
            call calc_tau_abs_mlay(band,quad,i+1,md,1,tau_B)
            dtauB_dxp=func_dtauB_dxp(i,md+1,p,dkadx,band,quad,ispecies)
            
c     dner(i,m+1)/dx(p), p=2,m
            if ((p.ge.0).and.(p.le.it-2)) then
               dNERis_dx_dir=0.0D+0
            else if ((p.ge.it-1).and.(p.le.m+1)) then
               dNERis_dx_dir=2.0D+0*pi*(-dtauB_dxp*(B_esp-B2)*
     &              expint(2,tau_B)+(dtauB_dxp+dtaui_dxp)*(B_esp-B1)*
     &              expint(2,tau_B+taui)+(B2-B1)/taui**2*(taui*(
     &              -dtauB_dxp*expint(3,tau_B)+
     &              (dtauB_dxp+dtaui_dxp)*expint(3,tau_B+taui))-
     &              dtaui_dxp*(expint(4,tau_B)-
     &              expint(4,tau_B+taui)))) ! W/m/m/[x]
            endif
            call calc_tau_abs_mlay(band,quad,1,md,1,tauH)
            dtauH_dxp=func_dtauB_dxp(0,md+1,p,dkadx,band,quad,ispecies)
            call calc_tau_abs_mlay(band,quad,1,i-1,1,tau_below)
            dtau_below_dxp=func_dtauB_dxp(0,i,p,dkadx,
     &           band,quad,ispecies)

            if (ref.eq.1) then ! specular reflexion
               dNERis_dx_ref=2.0D+0*pi*(1.0D+0-em_sol(band))*(
     &              -(dtau_below_dxp+dtauH_dxp)*(B_esp-B1)
     &              *expint(2,tau_below+tauH)
     &              +(dtau_below_dxp+dtauH_dxp+dtaui_dxp)*
     &              expint(2,tau_below+tauH+taui)
     &              -(B2-B1)/taui**2*(
     &              -taui*((dtau_below_dxp+dtauH_dxp)
     &              *expint(3,tau_below+tauH)
     &              -(dtau_below_dxp+dtauH_dxp+dtaui_dxp)
     &              *expint(3,tau_below+tauH+taui))
     &              -dtaui_dxp*(expint(4,tau_below+tauH)
     &              -expint(4,tau_below+tauH+taui))
     &              )
     &              ) ! W/m/m/[x]
            else if (ref.eq.2) then ! diffuse reflexion
               dNERis_dx_ref=4.0D+0*pi*(1.0D+0-em_sol(band))*(
     &              (-dtau_below_dxp*(B_esp-B1)*expint(2,tau_below)
     &              +(dtau_below_dxp+dtaui_dxp)*(B_esp-B2)
     &              *expint(2,tau_below+taui))
     &              -(B2-B1)/taui**2*(
     &              -taui*(dtau_below_dxp*expint(3,tau_below)
     &              -(dtau_below_dxp+dtaui_dxp)*expint(3,tau_below+taui)
     &              )
     &              -dtaui_dxp*(expint(4,tau_below)
     &              -expint(4,tau_below+taui))
     &              )*expint(3,tauH)
     &              -dtauH_dxp*(
     &              (B_esp-B1)*expint(3,tau_below)
     &              -(B_esp-B2)*expint(3,tau_below+taui)
     &              -(B2-B1)/taui*(expint(4,tau_below)-
     &              expint(4,tau_below+taui))
     &              )*expint(2,tauH)
     &              ) ! W/m/m/[x]
            endif
            
            dnerd_dx(i,md+1,p)=dNERis_dx_dir+dNERis_dx_ref
            
c     Computation of NER(i,j)
            do j=i+1,md
               jt=j/2+1
               z3=alti(j-1)
               z4=alti(j)
               call find_B(band,quad,z3,B3)
               call find_B(band,quad,z4,B4)
               call find_ka(band,quad,jt,(z4+z3)/2.0D+0,kaj)
               tauj=kaj*(z4-z3)
               dtauj_dxp=func_dtaui_dxp(j,p,dkadx,band,quad,ispecies)
               call calc_tau_abs_mlay(band,quad,i+1,j-1,1,tau_B)
               dtauB_dxp=func_dtauB_dxp(i,j,p,dkadx,band,quad,ispecies)

c     Direct d(NERd(i,j))/dx(p)
               if ((p.ge.1).and.(p.le.it-2)) then
                  dNERij_dxp_dir=0.0D+0
               else if ((p.ge.it-1).and.(p.le.jt+1)) then
                  call dNERij_dx_direct(B1,B2,B3,B4,
     &                 taui,tauj,tau_B,
     &                 dtaui_dxp,dtauj_dxp,dtauB_dxp,
     &                 dNERij_dxp_dir) ! dNERij_dxp_dir= W/m/m/[x]
               else if ((p.ge.jt+2).and.(p.le.m)) then
                  dNERij_dxp_dir=0.0D+0
               endif            ! p
               
c     Indirect d(NERd(i,j))/dx(p)
               dNERij_dxp_ref=0.0D+0
               if (p.le.jt+1) then
c     Both "dNERij_dxp_specular_reflection_virtual" and "dNERij_dxp_specular_reflection_real"
c     can be used for the computation of 'dNERij_dxp_ref'. They take the same
c     arguments, and the result will be the same. See comments in those two routines.

                  if (ref.eq.1) then ! specular reflection at ground level
c                     call dNERij_dx_specular_reflection_virtual(
c     &                    em_sol(band),
c     &                    B1,B2,B3,B4,
c     &                    tau_inf,tau_B,taui,tauj,
c     &                    dtau_inf_dxp,dtauB_dxp,
c     &                    dtaui_dxp,dtauj_dxp,
c     &                    dNERij_dxp_ref) ! dNERij_dxp_ref= W/m/m/[x]
                     call dNERij_dx_specular_reflection_real(
     &                    em_sol(band),
     &                    B1,B2,B3,B4,
     &                    tau_inf,tau_B,taui,tauj,
     &                    dtau_inf_dxp,dtauB_dxp,
     &                    dtaui_dxp,dtauj_dxp,
     &                    dNERij_dxp_ref)

                  else if (ref.eq.2) then ! diffuse reflection at ground level
                     call dNERij_dx_diffuse_reflection(
     &                    em_sol(band),
     &                    B1,B2,B3,B4,
     &                    tau_inf,tau_B,taui,tauj,
     &                    dtau_inf_dxp,dtauB_dxp,
     &                    dtaui_dxp,dtauj_dxp,
     &                    dNERij_dxp_ref)

                  endif         ! over ref

               else if ((p.ge.jt+2).and.(p.le.m)) then
                  dNERij_dxp_ref=0.0D+0
               endif            ! p
               
c     Sum of both direct and indirect
               dnerd_dx(i,j,p)=dNERij_dxp_dir+dNERij_dxp_ref ! W/m/m/[x]

            enddo               ! over j
         enddo                  ! over i

c     Computation of NER(ground-space)
         call calc_tau_abs_mlay(band,quad,1,md,1,tau_atmos)
c     Sensitivities of ground-space NER to x
         dnerd_dx(0,md+1,p)=2.0D+0*pi*(B_sol-B_esp)*(e(p)-e(p-1))
     &        *dkadx(band,quad,p,ispecies)
     &        *expint(2,tau_atmos) ! W/m2/m/[x]

c     dNERd_dx from grid "alti" combination in order to get dNER_dx for grid "e"

c     ner(ground-i)
         dner_dx(0,1,p)=dnerd_dx(0,1,p)
         do i=2,m-1
            dner_dx(0,i,p)=dnerd_dx(0,2*(i-1),p)+dnerd_dx(0,2*i-1,p)
         enddo
         dner_dx(0,m,p)=dnerd_dx(0,md,p)

c     ner(space-i)
         dner_dx(1,m+1,p)=dnerd_dx(1,md+1,p)
         do i=2,m-1
            dner_dx(i,m+1,p)=dnerd_dx(2*(i-1),md+1,p)+
     &           dnerd_dx(2*i-1,md+1,p)
         enddo
         dner_dx(m,m+1,p)=dnerd_dx(md,md+1,p)

c     ner(i,j)
         do j=2,m-1
            dner_dx(1,j,p)=dnerd_dx(1,2*(j-1),p)+dnerd_dx(1,2*j-1,p)
         enddo
         dner_dx(1,m,p)=dnerd_dx(1,md,p)
         do i=2,m-1
            do j=i+1,m-1
               dner_dx(i,j,p)=dnerd_dx(2*(i-1),2*(j-1),p)+
     &              dnerd_dx(2*(i-1),2*j-1,p)+
     &              dnerd_dx(2*i-1,2*(j-1),p)+
     &              dnerd_dx(2*i-1,2*j-1,p)
            enddo               ! j
         enddo                  ! i
         do i=2,m-1
            dner_dx(i,m,p)=dnerd_dx(2*(i-1),md,p)+dnerd_dx(2*i-1,md,p)
         enddo

c     ner(ground-space)
         dner_dx(0,m+1,p)=dnerd_dx(0,md+1,p)

c     -- end of recombination
         
c     Symetrization
         do i=1,m
            do j=1,i-1
               dner_dx(i,j,p)=-dner_dx(j,i,p)
            enddo
            dner_dx(i,i,p)=0.0D+0
            dner_dx(i,0,p)=-dner_dx(0,i,p)
            dner_dx(m+1,i,p)=-dner_dx(i,m+1,p)
         enddo
         dner_dx(0,0,p)=0.0D+0
         dner_dx(m+1,m+1,p)=0.0D+0
         dner_dx(m+1,0,p)=-dner_dx(0,m+1,p)    
         
      enddo                     ! p

      do p=i_max+1,m
         do i=1,md
            dnerd_dx(0,i,p)=0.0D+0
            dnerd_dx(i,md+1,p)=0.0D+0
            do j=i+1,md
               dnerd_dx(i,j,p)=0.0D+0
            enddo ! j
         enddo ! i
         dnerd_dx(0,md+1,p)=0.0D+0
      enddo ! p


 123  continue
      return
      end
c     ...sensitivities






c     sensitivities...
      double precision function func_dtaui_dxp(i,p,dkadx,band,quad,
     &     ispecies)
      implicit none
      include 'max.inc'
      include 'data.inc'
      integer i,p,it,band,quad,ispecies
      double precision z1,z2
      double precision dkadx(1:Nbmx,1:Nqmx,1:Nmax,1:Nsmx)
      
      it=i/2+1
      if (p.eq.it) then
         z1=alti(i-1)
         z2=alti(i)
         func_dtaui_dxp=dkadx(band,quad,it,ispecies)*(z2-z1)
      else
         func_dtaui_dxp=0.0D+0
      endif

      return
      end
c     ...sensitivities




c     sensitivities...
      double precision function func_dtauB_dxp(i,j,p,dkadx,
     &     band,quad,ispecies)
      implicit none
      include 'max.inc'
      include 'data.inc'
      integer i,j,p,it,jt,band,quad,ispecies,md
      double precision dkadx(1:Nbmx,1:Nqmx,1:Nmax,1:Nsmx)
      double precision deltaz

      it=i/2+1
      jt=j/2+1
      md=2*m-2
      if (j.le.i+1) then
         func_dtauB_dxp=0.0D+0
         goto 123
      endif

      if ((i.eq.0).and.(j.gt.1)) then
         if (p.eq.0) then
            deltaz=0.0D+0
         else if ((p.gt.0).and.(p.lt.jt)) then
            deltaz=e(p)-e(p-1)
         else if (p.eq.jt) then
            if (dble(j)/2.0D+0.eq.dble(j/2)) then ! for 'j' even
               deltaz=0.0D+0
            else                ! for 'j' odd
               deltaz=(alt(jt-1)-e(jt-1))
            endif 
         else
            deltaz=0.0D+0
         endif
      else if ((i.ge.1).and.(j.eq.md+1)) then
         if (p.lt.it) then
            deltaz=0.0D+0
         else if (p.eq.it) then
            if (dble(i)/2.0D+0.eq.dble(i/2)) then ! for 'i' even
               deltaz=(e(it)-alt(it-1))
            else                ! for 'i' odd
               deltaz=0.0D+0
            endif 
         else if (p.gt.it) then
            deltaz=e(p)-e(p-1)
         endif
      else if ((i.gt.0).and.(i.lt.md+1).and.(j.gt.0).and.(j.lt.md+1))
     &        then
         if (p.lt.it) then
            deltaz=0.0D+0
         else if (p.eq.it) then
            if (dble(i)/2.0D+0.eq.dble(i/2)) then ! for 'i' even
               deltaz=(e(it)-alt(it-1))
            else                ! for 'i' odd
               deltaz=0.0D+0
            endif
         else if ((p.gt.it).and.(p.lt.jt)) then
            deltaz=e(p)-e(p-1)
         else if (p.eq.jt) then
            if (dble(j)/2.0D+0.eq.dble(j/2)) then ! for 'j' even
               deltaz=0.0D+0
            else                ! for 'i' odd
               deltaz=(alt(jt-1)-e(jt-1))
            endif
         else if (p.gt.jt) then
            deltaz=0.0D+0
         endif
      else
         write(*,*) 'Could not compute dtauB_dxp'
         write(*,*) 'i=',i,' j=',j,' p=',p
         stop
      endif
         

      func_dtauB_dxp=deltaz*dkadx(band,quad,p,ispecies)
 123  continue
      return
      end
c     ...sensitivities




c     sensitivities...
      subroutine dNERij_dx_direct(B1,B2,B3,B4,
     &     taui,tauj,tau_B,
     &     dtaui_dxp,dtauj_dxp,dtauB_dxp,
     &     dNERij_dx_dir)
      implicit none
      include 'param.inc'
c     Sensitivity of the direct part of NER(i,j) to a chemical species concentration.
      
      double precision expint
      double precision B1,B2,B3,B4
      double precision taui,tauj,tau_B
      double precision dtaui_dxp,dtauj_dxp,dtauB_dxp
      double precision alpha,beta
      double precision dNERij_dx_dir


      alpha=expint(4,tau_B+taui)+expint(4,tau_B+tauj)
     &     -expint(4,tau_B)-expint(4,tau_B+taui+tauj)
      beta=-(dtauB_dxp+dtaui_dxp)*expint(3,tau_B+taui)
     &     -(dtauB_dxp+dtauj_dxp)*expint(3,tau_B+tauj)
     &     +dtauB_dxp*expint(3,tau_B)
     &     +(dtauB_dxp+dtaui_dxp+dtauj_dxp)*
     &     expint(3,tau_B+taui+tauj)
      dNERij_dx_dir=-2.0D+0*pi*(
     &     -dtauB_dxp*(B2-B3)*expint(2,tau_B)+
     &     (dtauB_dxp+dtaui_dxp)*(B1-B3)*
     &     expint(2,tau_B+taui)+
     &     (dtauB_dxp+dtauj_dxp)*(B2-B4)*
     &     expint(2,tau_B+tauj)-
     &     (dtauB_dxp+dtaui_dxp+dtauj_dxp)*(B1-B4)*
     &     expint(2,tau_B+taui+tauj)+
     &     (B2-B1)/taui**2*(taui*beta-
     &     dtaui_dxp*alpha)+
     &     (B4-B3)/tauj**2*(tauj*beta-
     &     dtauj_dxp*alpha))    ! W/m/m/[x]


      return
      end
c     ...sensitivities



c     sensitivities...
      subroutine dNERij_dx_specular_reflection_virtual(epsilon,
     &     B1,B2,B3,B4,
     &     tau_inf,tau_B,taui,tauj,dtau_inf_dxp,dtauB_dxp,
     &     dtaui_dxp,dtauj_dxp,
     &     dNERij_dxp_ref)
      implicit none
      include 'param.inc'
c     Sensitivity of the NER between layers (i) and (j) via specular ground reflection
c     to a chemical species concentration. This routine will use a virtual layer (-i)
c     that is the symetrical of layer (i) with respect to the ground.

      double precision expint,epsilon
      double precision B1,B2,B3,B4
      double precision tau_inf,tau_B,taui,tauj
      double precision dtau_inf_dxp,dtauB_dxp,dtaui_dxp,dtauj_dxp
      double precision B1_ind,B2_ind,B3_ind,B4_ind
      double precision tau_B_ind,taui_ind,tauj_ind
      double precision dtauB_dxp_ind,dtaui_dxp_ind,dtauj_dxp_ind
      double precision dNERij_dxp_dir,dNERij_dxp_ref


      B1_ind=B2
      B2_ind=B1
      B3_ind=B3
      B4_ind=B4
      taui_ind=taui
      tauj_ind=tauj
      tau_B_ind=2.0D+0*tau_inf+taui+tau_B
      dtaui_dxp_ind=dtaui_dxp
      dtauj_dxp_ind=dtauj_dxp
      dtauB_dxp_ind=2.0D+0*dtau_inf_dxp+dtaui_dxp+dtauB_dxp

      call dNERij_dx_direct(B1_ind,B2_ind,B3_ind,B4_ind,
     &     taui_ind,tauj_ind,tau_B_ind,
     &     dtaui_dxp_ind,dtauj_dxp_ind,dtauB_dxp_ind,
     &     dNERij_dxp_dir)

      dNERij_dxp_ref=(1.0D+0-epsilon)*dNERij_dxp_dir ! W/m/m/[x]

      return
      end
c     ...sensitivities




c     sensitivities...
      subroutine dNERij_dx_specular_reflection_real(epsilon,
     &     B1,B2,B3,B4,
     &     tau_inf,tau_B,taui,tauj,dtau_inf_dxp,dtauB_dxp,
     &     dtaui_dxp,dtauj_dxp,
     &     dNERij_dxp_ref)
      implicit none
      include 'param.inc'
c     Sensitivity of the NER between layers (i) and (j) via specular ground reflection
c     to a chemical species concentration. This routine uses the derived expression of
c     the reflected NER(i,j).
c     The result of the present routine is perfectly equivalent to the result
c     given by routine "dNERij_dTx_specular_reflection_virtual"

      double precision expint,epsilon
      double precision B1,B2,B3,B4
      double precision tau_inf,tau_B,taui,tauj
      double precision dtau_inf_dxp,dtauB_dxp,dtaui_dxp,dtauj_dxp
      double precision tau_B1,tau_B2,dtauB1_dxp,dtauB2_dxp
      double precision a(1:6)
      double precision da_dxp(1:6)
      double precision dNERij_dxp_ref

      tau_B1=tau_inf
      tau_B2=tau_B1+taui+tau_B
      dtauB1_dxp=dtau_inf_dxp
      dtauB2_dxp=dtauB1_dxp+dtaui_dxp+dtauB_dxp
      call a_coeffs(B1,B2,B3,B4,taui,tauj,a)
      call da_dxp_coeffs(B1,B2,B3,B4,
     &     taui,tauj,dtaui_dxp,dtauj_dxp,
     &     da_dxp)

      dNERij_dxp_ref=2.0D+0*pi*(1.0D+0-epsilon)*
     &     (
     &     da_dxp(1)*expint(3,tau_B1+tau_B2)
     &     -a(1)*(dtauB1_dxp+dtauB2_dxp)*expint(2,tau_B1+tau_B2)
     &     +(da_dxp(2)+da_dxp(4))*expint(4,tau_B1+tau_B2)
     &     -(a(2)+a(4))*(dtauB1_dxp+dtauB2_dxp)*expint(3,tau_B1+tau_B2)
     &     +da_dxp(3)*expint(3,tau_B1+tau_B2+taui)
     &     -a(3)*(dtauB1_dxp+dtauB2_dxp+dtaui_dxp)
     &     *expint(2,tau_B1+tau_B2+taui)
     &     -(da_dxp(2)+da_dxp(4))*expint(4,tau_B1+tau_B2+taui)
     &     +(a(2)+a(4))*(dtauB1_dxp+dtauB2_dxp+dtaui_dxp)
     &     *expint(3,tau_B1+tau_B2+taui)
     &     +da_dxp(5)*expint(3,tau_B1+tau_B2+tauj)
     &     -a(5)*(dtauB1_dxp+dtauB2_dxp+dtauj_dxp)
     &     *expint(2,tau_B1+tau_B2+tauj)
     &     -(da_dxp(2)+da_dxp(4))*expint(4,tau_B1+tau_B2+tauj)
     &     +(a(2)+a(4))*(dtauB1_dxp+dtauB2_dxp+dtauj_dxp)
     &     *expint(3,tau_B1+tau_B2+tauj)
     &     +da_dxp(6)*expint(3,tau_B1+tau_B2+taui+tauj)
     &     -a(6)*(dtauB1_dxp+dtauB2_dxp+dtaui_dxp+dtauj_dxp)
     &     *expint(2,tau_B1+tau_B2+taui+tauj)
     &     +(da_dxp(2)+da_dxp(4))*expint(4,tau_B1+tau_B2+taui+tauj)
     &     -(a(2)+a(4))*(dtauB1_dxp+dtauB2_dxp+dtaui_dxp+dtauj_dxp)
     &     *expint(3,tau_B1+tau_B2+taui+tauj)
     &     )                    ! dNERij_dxp_ref=W/m/m/[x]

      return
      end
c     ...sensitivities





c     sensitivities...
      subroutine dNERij_dx_diffuse_reflection(epsilon,
     &     B1,B2,B3,B4,
     &     tau_inf,tau_B,taui,tauj,dtau_inf_dxp,dtauB_dxp,
     &     dtaui_dxp,dtauj_dxp,
     &     dNERij_dxp_ref)
      implicit none
      include 'param.inc'
c     Sensitivity of the NER between layers (i) and (j) via diffuse ground reflection
c     to a chemical species concentration. 
      double precision expint,epsilon
      double precision B1,B2,B3,B4
      double precision tau_inf,tau_B,taui,tauj
      double precision dtau_inf_dxp,dtauB_dxp,dtaui_dxp,dtauj_dxp
      double precision tau_B1,tau_B2,dtauB1_dxp,dtauB2_dxp
      double precision alpha3,beta3,gamma3,delta3
      double precision dalpha3_dxp,dbeta3_dxp,dgamma3_dxp,ddelta3_dxp
      double precision dNERij_dxp_ref

      tau_B1=tau_inf
      tau_B2=tau_B1+taui+tau_B
      dtauB1_dxp=dtau_inf_dxp
      dtauB2_dxp=dtauB1_dxp+dtaui_dxp+dtauB_dxp
      call coeffs3(B1,B2,B3,B4,taui,tauj,tau_B1,tau_B2,
     &     alpha3,beta3,gamma3,delta3)

      call dcoeffs3_dxp(B1,B2,B3,B4,taui,tauj,tau_B1,tau_B2,
     &     dtaui_dxp,dtauj_dxp,dtauB1_dxp,dtauB2_dxp,
     &     dalpha3_dxp,dbeta3_dxp,dgamma3_dxp,ddelta3_dxp)

      dNERij_dxp_ref=4.0D+00*pi*(1.0D+0-epsilon)*
     &     (
     &     dalpha3_dxp*expint(3,tau_B1)
     &     -alpha3*dtauB1_dxp*expint(2,tau_B1)
     &     +dbeta3_dxp*expint(4,tau_B1)
     &     -beta3*dtauB1_dxp*expint(3,tau_B1)
     &     +dgamma3_dxp*expint(3,tau_B1+taui)
     &     -gamma3*(dtauB1_dxp+dtaui_dxp)*expint(2,tau_B1+taui)
     &     +ddelta3_dxp*expint(4,tau_B1+taui)
     &     -delta3*(dtauB1_dxp+dtaui_dxp)*expint(3,tau_B1+taui)
     &     )                    ! dNERij_dxp_ref=W/m/m/[x}

      return
      end
c     ...sensitivities



c     sensitivities...
      subroutine da_dxp_coeffs(B1,B2,B3,B4,
     &     taui,tauj,dtaui_dxp,dtauj_dxp,
     &     da_dxp)
      implicit none
      double precision B1,B2,B3,B4
      double precision taui,tauj
      double precision dB1_dxp,dB2_dxp,dB3_dxp,dB4_dxp
      double precision dtaui_dxp,dtauj_dxp
      double precision da_dxp(1:6)

      da_dxp(1)=0.0D+0
      da_dxp(2)=dtaui_dxp*(B2-B1)/taui**2
      da_dxp(3)=0.0D+0
      da_dxp(4)=dtauj_dxp*(B3-B4)/tauj**2
      da_dxp(5)=0.0D+0
      da_dxp(6)=0.0D+0

      return
      end
c     ...sensitivities



c     sensitivities...
      subroutine dcoeffs3_dxp(B1,B2,B3,B4,taui,tauj,tau_B1,tau_B2,
     &     dtaui_dxp,dtauj_dxp,dtauB1_dxp,dtauB2_dxp,
     &     dalpha3_dxp,dbeta3_dxp,dgamma3_dxp,ddelta3_dxp)
      implicit none
      double precision expint
      double precision B1,B2,B3,B4,taui,tauj,tau_B1,tau_B2
      double precision dtaui_dxp,dtauj_dxp
      double precision dtauB1_dxp,dtauB2_dxp
      double precision a(1:6)
      double precision da_dxp(1:6)
      double precision dalpha3_dxp,dbeta3_dxp,dgamma3_dxp,ddelta3_dxp

      call a_coeffs(B1,B2,B3,B4,taui,tauj,a)

      call da_dxp_coeffs(B1,B2,B3,B4,taui,tauj,
     &     dtaui_dxp,dtauj_dxp,da_dxp)

      dalpha3_dxp=da_dxp(1)*expint(3,tau_B2)
     &     -a(1)*dtauB2_dxp*expint(2,tau_B2)
     &     +da_dxp(4)*expint(4,tau_B2)
     &     -a(4)*dtauB2_dxp*expint(3,tau_B2)
     &     +da_dxp(5)*expint(3,tau_B2+tauj)
     &     -a(5)*(dtauB2_dxp+dtauj_dxp)*expint(2,tau_B2+tauj)
     &     -da_dxp(4)*expint(4,tau_B2+tauj)
     &     +a(4)*(dtauB2_dxp+dtauj_dxp)*expint(3,tau_B2+tauj)

      dbeta3_dxp=da_dxp(2)*(expint(3,tau_B2)-expint(3,tau_B2+tauj))
     &     -a(2)*(dtauB2_dxp*expint(2,tau_B2)-
     &     (dtauB2_dxp+dtauj_dxp)*expint(2,tau_B2+tauj))

      dgamma3_dxp=da_dxp(3)*expint(3,tau_B2)
     &     -a(3)*dtauB2_dxp*expint(2,tau_B2)
     &     -da_dxp(4)*expint(4,tau_B2)
     &     +a(4)*dtauB2_dxp*expint(3,tau_B2)
     &     +da_dxp(6)*expint(3,tau_B2+tauj)
     &     -a(6)*(dtauB2_dxp+dtauj_dxp)*expint(2,tau_B2+tauj)
     &     +da_dxp(4)*expint(4,tau_B2+tauj)
     &     -a(4)*(dtauB2_dxp+dtauj_dxp)*expint(3,tau_B2+tauj)

      ddelta3_dxp=-dbeta3_dxp

      return
      end
c     ...sensitivities
