c     Copyright (C) 2000-2008
c     Universite Paul Sabatier, Toulouse, France

c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine initz1(j,sens,band,quad,DeltaXmax,w_in,B_em,
     &     alpha_out,w_out,ksi_out,arret)

      implicit none

c     Cette routine cre un chemin de diffusion complet dans la maille d'mission j
c     suivant le processus d'mission numro 1 : choix d'un angle de sortie de maille,
c     puis choix d'un point de dpart sur un segment droit inclin de cet angle,
c     et enfin gnration du chemin de diffuion jusqu' premire sortie de la maille
c     
c     Entres :
c       + j : numro de la maille gazeuse d'mission
c       + sens : sens=0 : tir vers le bas ; sens=1 : tir vers le haut
c       + band : numro de la bande troite
c       + DeltaXmax : delta T maximum dans l'atmosphre
c       + quad : numro de la quadrature
c       + w_in : poids du rayon  l'mission (choix frquentiel)
c
c     Sortie :
c       + B_em : luminance du point d'mission du rayon
c       + alpha_out : angle de sortie de la maille
c       + w_out : poids de sortie du photon -- combine pdf angulaire et pdf(tau_abs)
c       + ksi_out : nergie restante aprs diffusion dans la maille
c       + arret : si arret=1, le chemin de diffusion a t tronqu ; sinon arret=0
c

      include 'max.inc'
      include 'data.inc'
      include 'options.inc'
      include 'param.inc'
      integer j,out,r_max,arret,p,sens,nd,band,quad
      double precision w_in,DeltaXmax
      double precision alpha_out,w_ang,w_em,w_out,raf1,ksi_min,ksi_out
      double precision tau_abs,alpha,w,pdf,pdf0,B_em,tau_chemin,tau_out
      double precision tau_diff,g
      double precision tau_max,ray(0:Diffmax,2),long,z0,z1,alpha0,wo,raf

      ray(0,1)=0.0D+0
      ray(0,2)=0.0D+0

c     Choix d'un angle de sortie de maille et calcul du poids angulaire
      ray(1,1)=e(j-1)
      ray(1,2)=0.0D+0
      ray(2,1)=e(j)
      ray(2,2)=e(j)-e(j-1)
      call find_tau_abs(band,quad,j,ray,2,tau_abs) ! Calcul de l'ep. opt. d'abs. de la maille j
      call find_tau_diff(band,quad,j,ray,2,tau_diff) ! Calcul de l'ep. opt. de diff. de la maille j
      call find_g(band,quad,j,(e(j)+e(j-1))/2.0D+0,g) ! Calcul du g pour la maille j
      call pdf_angulaire(1,sens,1,raf1,tau_abs,tau_diff,g,alpha,
     &     pdf0,pdf)            ! Calcul de l'angle et de son pdf
      w_ang=pdf0/pdf
      alpha0=alpha
      
c     Dtermination de l'altitude de dpart du rayon sur le segment
                                ! Calcul de l'ep. opt. d'abs. du segment 'tau_max'
      ray(2,2)=(e(j)-e(j-1))/dabs(dcos(alpha))
      call  find_tau_abs(band,quad,j,ray,2,tau_max)
      call  pdf_tau_abs(1,tau_max,raf1,tau_out,pdf) ! Calcul de l'ep. opt. d'abs. du pt de dpart
      w_em=1.D+0/pdf
      if (alpha.lt.pi/2.D+0) then ! Calcul de z0, l'altitude de dpart du segment
         z0=e(j-1)
      else
         z0=e(j)
      endif
      call find_z1_abs(band,quad,j,z0,pi-alpha,tau_out,z1) ! Calcul de z1, l'altitude d'mission du rayon
      if (z1.lt.0.0D+0) then
         call screen(1108,0,0,0.0D+0,0.0D+0)
         stop
      endif

c     Calcul de la luminance du point d'mission du rayon
      call find_B(band,quad,z1,B_em)
      call calc_stop(band,quad,DeltaXmax,B_em,ksi_min)

c     Calcul du chemin de diffusion jusqu' premire sortie de la maille j
      call path(band,quad,j,z1,alpha,1.0D+0,ksi_min,ray,r_max,nd,
     &     alpha_out,arret)
c     Rtablir le sens de propagation du rayon
      if (arret.eq.0) then
         if (alpha_out.le.pi/2.0D+0) then
            sens=0
         else
            sens=1
         endif
      endif

      call countd(0,j,nd,ray(r_max,2))

c     Calcul de l'absorption sur la longueur du chemin gnr
      call find_tau_abs(band,quad,j,ray,r_max,tau_chemin)
      ksi_out=dexp(-tau_chemin)
      
c     Calcul du poids total du photon  la sortie de la maille d'mission
      if (tirpg.eq.1) then
         wo=1.0D+0
      else
         wo=pi
      endif
      w_out=w_in*wo*w_ang*w_em

30000 continue
      return
      end
