c     Copyright (C) 2000-2008
c     Universite Paul Sabatier, Toulouse, France

c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine ksi_matrix(r,band,T0,pne,ksi)
      implicit none
      include 'max.inc'
      include 'data.inc'
      include 'param.inc'

      integer i,j,band,res,r,nlay
      double precision lambda1,nbw
      double precision T0(0:Nmax+1),Ti,Tj
      double precision Bi,Bj,Planck
      double precision pne(0:Nmax+1,0:Nmax+1)
      double precision ksi(0:Nmax+1,0:Nmax+1)
      double precision ksi_min
      parameter(ksi_min=1.0D-60)

c     Purpose: to compute the matrix of 
c     ksi(i,j,band)=ner(i,j,band)/[B(i,band)-B(j,band)]
c     for a given band
c
c     r=0: use non-recombined grid
c     r=1: use recombined grid

      if (r.eq.0) then
         nlay=m
      else
         nlay=m-2
      endif

c     wavelength at the center of the narrowband
      lambda1=(lambda_lo(band)+lambda_hi(band))/2.0D+0 ! m
      nbw=1.0D+4/lambda_lo(band)-1.0D+4/lambda_hi(band) ! cm

c     Computation of ksi
      do i=0,nlay+1
c     pne(i,j) is given in W/m

c     Debug
         do j=0,nlay+1
            call test_nan(pne(i,j),res)
            if (res.eq.1) then
               call screen(1131,i,j,dble(band),pne(i,j))
               stop
            endif
         enddo
c     Debug

         Bi=Planck(T0(i),lambda_lo(band),lambda_hi(band)) ! W/m/sr/m
         do j=0,nlay+1
            Bj=Planck(T0(j),lambda_lo(band),lambda_hi(band)) ! W/m/sr/m
            if ((j.eq.i).or.(Bi.eq.Bj)) then
               ksi(i,j)=0.0D+0
            else
               ksi(i,j)=pne(i,j)/nbw/(Bi-Bj) ! sr.m/cm
               if (dabs(ksi(i,j)).lt.ksi_min) then
                  ksi(i,j)=0.0D+0
               endif
            endif
         enddo ! j
c     ksi(i,j) is saved in [W/m/cm]/[W/m/sr/m]=sr.m/cm
c     In order to re-compute a NER matric over a narrowband 'band',
c     one has to multiply ksi(i,j,band) by (B(i,band)-B(j,band)) using
c     the same convention, i.e. B(i,band) and B(j,band) in W/m/sr/m,
c     which gives psi(i,j,band) in W/m/cm.
c     Next, in order to compute a ner(i,j) matrix over the whole
c     spectrum (W/m), one has to sum all ner(i,j,band) matrices
c     multiplied by the narrowband width in cm
      enddo ! i

      return
      end
