c     Copyright (C) 2000-2008
c     Universite Paul Sabatier, Toulouse, France

c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine path(band,quad,j,z0,alpha_in,ksi_in,ksi_min,
     &     ray,r_max,nd,alpha_out,arret)
      implicit none
      include 'max.inc'
      include 'data.inc'
      include 'options.inc'
      include 'param.inc'
      
c     Cette subroutine permet de calculer un chemin de diffusion complet dans une maille
c
c     Entres:
c       + j        : indice de la maille o ce chemin est gnr
c       + z0       : altitude de dpart du rayon
c       + alpha_in : angle de dpart du rayon
c       + ksi_in   : poids du rayon au dbut du chemin
c       + ksi_min  : poids minimal permettant d'arrter le chemin du rayon
c
c
c     Sorties:
c       + ray       : tableau Ray correspondant au trajet du rayon
c        - ray(*,1) : altitudes des points de diffusion du rayon
c        - ray(*,2) : longueur de diffusion cumule
c       + r_max     : indice reprant la fin du tableau ray
c       + nd        : nombre de diffusions subies dans la maille de gaz
c       + alpha_out : angle du rayon au dernier point du chemin
c       + arret     : si =1, on a tronqu le chemin ; si =0, le chemin est complet
c

      double precision ray(0:DiffMax,2),alpha_in,z0,z,z1,w0,ksi_min,ksi
      integer j,r,r_max,arret,nd,band,quad
      double precision alpha_out,ld,phi,TauR,TauRr
      double precision alpha,alphar,raf1,raf2,ksi_in
      double precision ray2(0:Diffmax,2),tau_abs,tau_diff

      ray(0,1)=0.D+0
      ray(0,2)=0.D+0
      z=z0
      ray(1,1)=z0
      ray(1,2)=0.D+0
      r=1
      alpha=alpha_in
      ksi=ksi_in
      arret=0
      nd=0

      call pdf_tau_diff(1,raf1,TauR,raf2)

10000 continue

c     Debug
      if (ray(r,1).lt.0.0D+0) then
         write(*,*) 'ray(',r,',1):',ray(r,1)
         write(*,*) 'tau_abs:',tau_abs
         stop
      endif
c     Debug

      call find_z1_diff(band,quad,j,ray(r,1),alpha,TauR,z1)
c     Debug
c      write(*,*) 'z1:',z1
c      write(*,*) 'j:',j,' ray(',r,',1):',ray(r,1)
c      write(*,*) 'alpha:',alpha,' TauR:',TauR
c     Debug

      if (r.gt.DiffMax-1) then ! Si DiffMax trop petit
         call screen(1104,0,0,0.0D+0,0.0D+0)
         stop
      endif
      
      if ((atd.eq.1).and.(r.gt.Ndiff)) then
         goto 30000 ! Si on a dcid d'arrter des trajectoires
      endif ! au bout de Ndiff diffusions

      if (alpha.gt.pi/2.D+0) then ! Si le rayon monte
         if (z1.lt.e(j)) then ! Si le rayon diffuse dans la maille
            nd=nd+1
            ray(r+1,1)=z1
            ray(r+1,2)=ray(r,2)+(ray(r,1)-ray(r+1,1))/dcos(alpha)
            ray2(1,1)=ray(r,1)
            ray2(1,2)=0.0D+0
            ray2(2,1)=ray(r+1,1)
            ray2(2,2)=ray(r+1,2)-ray(r,2)
            call find_tau_abs(band,quad,j,ray2,2,tau_abs)
            call find_tau_diff(band,quad,j,ray2,2,tau_diff)
c            ksi=ksi*dexp(-tau_abs-tau_diff)
            ksi=ksi*dexp(-tau_abs)
            r=r+1
            if (ksi.lt.ksi_min) then ! Test pour vrifier si on tronque ou pas
               arret=1
               goto 20000
            endif
            call scatter(band,quad,j,ray(r,1),alpha,alphar,phi,TauRr)
            alpha=alphar
            TauR=TauRr
            goto 10000 ! On continue
         else ! Sinon le rayon sort de la maille par le haut
            ray(r+1,1)=e(j)
            ray(r+1,2)=ray(r,2)+(ray(r,1)-ray(r+1,1))/dcos(alpha)
            ray2(1,1)=ray(r,1)
            ray2(1,2)=0.0D+0
            ray2(2,1)=ray(r+1,1)
            ray2(2,2)=ray(r+1,2)-ray(r,2)
            call find_tau_abs(band,quad,j,ray2,2,tau_abs)
            call find_tau_diff(band,quad,j,ray2,2,tau_diff)
c            ksi=ksi*dexp(-tau_abs-tau_diff)
            ksi=ksi*dexp(-tau_abs)
            r=r+1
            goto 20000 ! Fin du chemin
         endif
      else ! Sinon le rayon descend
         if (z1.gt.e(j-1)) then ! Si le rayon diffuse dans la maille
            nd=nd+1
            ray(r+1,1)=z1
            ray(r+1,2)=ray(r,2)+(ray(r,1)-ray(r+1,1))/dcos(alpha)
            ray2(1,1)=ray(r,1)
            ray2(1,2)=0.0D+0
            ray2(2,1)=ray(r+1,1)
            ray2(2,2)=ray(r+1,2)-ray(r,2)
            call find_tau_abs(band,quad,j,ray2,2,tau_abs)
            call find_tau_diff(band,quad,j,ray2,2,tau_diff)
c            ksi=ksi*dexp(-tau_abs-tau_diff)
            ksi=ksi*dexp(-tau_abs)
            r=r+1
            if (ksi.lt.ksi_min) then ! Test pour vrifier si on tronque ou pas
               arret=1
               goto 20000
            endif
            call scatter(band,quad,j,ray(r,1),alpha,alphar,phi,TauRr)
            alpha=alphar
            TauR=TauRr
            goto 10000 ! On continue
         else ! Sinon le rayon sort de la maille par le bas
            ray(r+1,1)=e(j-1)
            ray(r+1,2)=ray(r,2)+(ray(r,1)-ray(r+1,1))/dcos(alpha)
            ray2(1,1)=ray(r,1)
            ray2(1,2)=0.0D+0
            ray2(2,1)=ray(r+1,1)
            ray2(2,2)=ray(r+1,2)-ray(r,2)
            call find_tau_abs(band,quad,j,ray2,2,tau_abs)
            call find_tau_diff(band,quad,j,ray2,2,tau_diff)
c            ksi=ksi*dexp(-tau_abs-tau_diff)
            ksi=ksi*dexp(-tau_abs)
            r=r+1
            goto 20000 ! Fin du chemin
         endif
      endif
      
20000 continue

      r_max=r
      alpha_out=alpha

c     Debug
c      do r=1,r_max
c         write(*,*) 'ray(',r,',1):',ray(r,1),
c     &        ' ray(',r,',2):',ray(r,2)
c      enddo
c     Debug

30000 continue

      return
      end
