c     Copyright (C) 2000-2008
c     Universite Paul Sabatier, Toulouse, France

c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine pdf_angulaire(gen,sens,pog,alpha_in,tau_abs_in,
     &     tau_diff_in,g,alpha_out,pdf0,pdf)

c     Calcul de l'angle theta suivant une pdf optimise
c
c     Entrees :
c       + gen : si gen=0, on ne gnre pas d'angle suivant le pdf spcifi
c               si gen=1, on gnre un angle  suivant le pdf spcifi
c       + sens : sens=0 : tir vers le bas ; sens=1 : tir vers le haut
c       + pog : si pog=0, on tire cet angle sur une paroi
c               si pog=1, on tire cet angle dans une maille gazeuse
c       + alpha_in : angle duquel on calcule le pdf si gen=0
c       + tau_abs_in : epaisseur optique d'absorption de la couche pour un tirage en maille gazeuse
c       + tau_diff_in : epaisseur optique de diffusion de la couche pour un tirage en maille gazeuse
c       + g : facteur d'asymtrie pour tirage en maille gazeuse
c
c     Sorties :
c       + alpha_out : angle gnr suivant le pdf choisi dans les options
c       + pdf0 : pdf 'vrai' suivant la physique
c       + pdf : pdf choisi pour le tirage
c
c     Note : les pdf angulaires sont grs par les routines "pdf_ang*"
      
      implicit none
      include 'max.inc'
      include 'param.inc'
      include 'options.inc'
      
      double precision alpha_in,alpha_out,alpha
      double precision R,pdf0,pdf,tau_diff_in,g
      integer gen,pog,sens,out
      double precision u
      double precision tau_abs_in,raf1,raf2

      alpha=alpha_in
      
      if (pog.eq.0) then ! Si on tire l'angle sur une paroi
         if ((tirpp.eq.1).and.(lambertp.eq.0).and.(isop.eq.0)) then
c     Tirage perpendiculaire aux parois
            call pdf_ang_fixe(gen,sens,0,alpha,pdf)
            alpha_out=alpha
         else if ((tirpp.eq.0).and.(lambertp.eq.1).and.(isop.eq.0)) then
c     Tirage lambertien en luminance
            call pdf_ang_lambert(gen,sens,0,alpha,pdf)
            alpha_out=alpha
         else if ((tirpp.eq.0).and.(lambertp.eq.0).and.(isop.eq.1)) then
c     Tirage isotrope en luminance
            call pdf_ang_iso(gen,sens,0,alpha,pdf)
            alpha_out=alpha
         else if ((tirpp.eq.0).and.(lambertp.eq.0).and.(isop.eq.0)) then
c     Tirage optimis suivant l'ep. opt.
            call pdf_ang_opt(gen,sens,0,tau_abs_in,tau_diff_in,g,
     &           alpha,pdf)
            alpha_out=alpha
         endif
      else if (pog.eq.1) then ! Si on tire cet angle dans une maille gazeuse
         if ((tirpg.eq.1).and.(lambertg.eq.0).and.(isog.eq.0)) then
c     Tirage perpendiculaire aux parois
            call pdf_ang_fixe(gen,sens,1,alpha,pdf)
            alpha_out=alpha
         else if ((tirpg.eq.0).and.(lambertg.eq.1).and.(isog.eq.0)) then
c     Tirage lambertien en luminance
            call pdf_ang_lambert(gen,sens,1,alpha,pdf)
            alpha_out=alpha
         else if ((tirpg.eq.0).and.(lambertg.eq.0).and.(isog.eq.1)) then
c     Tirage isotrope en luminance
            call pdf_ang_iso(gen,sens,1,alpha,pdf)
            alpha_out=alpha
         else if ((tirpg.eq.0).and.(lambertg.eq.0).and.(isog.eq.0)) then
c     Tirage optimis suivant l'ep. opt.
            call pdf_ang_opt(gen,sens,1,tau_abs_in,tau_diff_in,g,
     &           alpha,pdf)
            alpha_out=alpha
         endif
      else ! Tirage d'angle ni sur une paroi ni dans le gaz -> ???
         out=1
         call screen(1103,0,0,0.0D+0,0.0D+0)
      endif

      call pdf_ang_lambert(0,sens,pog,alpha,pdf0)
      
      return
      end
      


      subroutine pdf_ang_fixe(gen,sens,pog,alpha,pdf)
      implicit none
      integer gen,pog,sens,out
      double precision alpha,pdf,R,pdfc
      include 'max.inc'
      include 'data.inc'
      include 'param.inc'
c     Routine de gnration d'un angle et de son pdf pour un tirage perpendiculaire aux parois
c
      pdfc=1.0D+0
      if (gen.eq.1) then
         if (pog.eq.0) then
            alpha=ang_tir_wall*pi/180.D+0
         else if (pog.eq.1) then
            alpha=ang_tir_gaz*pi/180.D+0
         else
            out=1
            call screen(1103,0,0,0.0D+0,0.0D+0)
         endif
c On ne tire alatoirement un angle vers le haut ou vers le bas que pour les mailles gazeuses
         if (pog.eq.1) then
            call choice_ud(alpha,sens,pdfc)
         endif
      endif
      pdf=1.0D+0/pi/pdfc
      return
      end



      subroutine pdf_ang_lambert(gen,sens,pog,alpha,pdf)
      implicit none
      integer gen,pog,sens
      double precision alpha,alpha_out,pdf,R,pdfc
      include 'param.inc'
c     Routine de gnration d'un angle et de son pdf pour un tirage suivant
c     une luminance isotrope -- intensit lambertienne
c     C'est le pdf vrai suivant la physique
c
      pdfc=1.0D+0
      if (gen.eq.1) then
         call rand_uniforme(R)
         alpha=dasin(dsqrt(R))
c On ne tire alatoirement un angle vers le haut ou vers le bas que pour les mailles gazeuses
         if (pog.eq.1) then
            call choice_ud(alpha,sens,pdfc)
         endif
      endif
      pdf=dabs(dcos(alpha))/pi/pdfc
      return
      end


      subroutine pdf_ang_iso(gen,sens,pog,alpha,pdf)
      implicit none
      integer gen,pog,sens
      double precision alpha,pdf,R,pdfc
      include 'param.inc'
c     Routine de gnration d'un angle et de son pdf pour un tirage suivant
c     une intensit isotrope
c
      pdfc=1.0D+0
      if (gen.eq.1) then
         call rand_uniforme(R)
         alpha=dacos(R)
c On ne tire alatoirement un angle vers le haut ou vers le bas que pour les mailles gazeuses
         if (pog.eq.1) then
            call choice_ud(alpha,sens,pdfc)
         endif
      endif
      pdf=1.D+0/(2.0D+0*pi)/pdfc
      return
      end


      subroutine pdf_ang_opt(gen,sens,pog,tau_abs_in,tau_diff_in,g,
     &     alpha,pdf)
      implicit none
      integer gen,pog,sens
      double precision alpha,pdf,a,b,u,rc,R,pdfc
      double precision tau_abs_in,tau_diff_in,g,tau_lim
      include 'param.inc'
      include 'options.inc'
c     Routine de gnration d'un angle et de son pdf pour un tirage
c     optimis suivant l'paisseur optique
c     
c     Entres :
c       + gen : 0 : on ne gnre pas d'angle (uniquement le pdf)
c               1 : on gnre un angle et son pdf
c       + sens : sens=0 : tir vers le bas ; sens=1 : tir vers le haut
c       + pog : 0 : en maille paroi
c               1 : en maille gaz
c       + tau_abs_in : paisseur optique de la maille gaz
c     
c     Sorties :
c       + alpha : angle
c       + pdf : son pdf
c
      if (pao.eq.1) then
         tau_lim=tau_abs_in
      else if (pao.eq.2) then
         tau_lim=tau_diff_in*(1.0D+0-g)+tau_abs_in
      endif
      pdfc=1.0D+0
      if (gen.eq.1) then
         call rand_uniforme(R)
         if (tau_lim.ge.1.d+0) then 
            a=1.d+0
            b=0.d+0
            alpha=dasin(dsqrt(R))
         else
            b=1.d+0/(1-tau_abs_in/2.d+0)
            a=b/(2*tau_abs_in)
            rc=a*tau_abs_in**2
            if (R.lt.rc) then
               u=dsqrt(2.d+0*R/(2.D+0*a))
            else
               u=(R-rc)/b+tau_abs_in
            endif
            alpha=dacos(u)
         endif

c On ne tire alatoirement un angle vers le haut ou vers le bas que pour les mailles gazeuses
         if (pog.eq.1) then
            call choice_ud(alpha,sens,pdfc)
         endif
      endif
c     Calcul du pdf optimis
      u=dabs(dcos(alpha))
      if (tau_lim.ge.1.D+0) then
         call  pdf_ang_lambert(gen,sens,pog,alpha,pdf)
      else
         b=1.d+0/(1-tau_abs_in/2.d+0)
         a=b/(2*tau_abs_in)
         if (u.le.tau_abs_in) then
            pdf=(a*u)/pi/pdfc
         else
            pdf=b/(2.D+0*pi)/pdfc
         endif
      endif

      return
      end


      subroutine pdf_ang_spec(alpha_in,alpha_out,pdf)
      implicit none
c     Gnration d'un angle de rflexion suivant une loi spculaire
      double precision alpha_in,alpha_out,pdf
      include 'param.inc'
      
      alpha_out=pi-alpha_in
      pdf=1.0D+0

      return
      end
