c     Copyright (C) 2000-2008
c     Universite Paul Sabatier, Toulouse, France

c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine scatter(band,quad,j,z,alpha_in,
     &     alpha_out,phi_out,tau_out)

      implicit none

c     Cette subroutine permet de traiter la diffusion sous l'hypothse d'une
c     fonction de phase de Henyey-Greenstein, pour la diffusion par
c     les nuages de particules (arosols, nuages H2O). Pour la diffusion
c     par le gaz, on utilise la fonction de phase de Rayleigh.
c
c     Les entres sont:
c         . band :  numro de bande troite
c         . quad :  numro de quadrature
c         . j : numro de la maille o se droule la diffusion
c         . z : altitude courante dans l'atmosphre
c         . alpha_in : angle de propagation avant diffusion
c
c     Sorties:
c         . alpha_out : angle de propagation aprs diffusion
c         . phi_out : angle phi, uniforme
c         . tau_out : prochaine paisseur optique de diffusion
c

      include 'max.inc'
      include 'param.inc'
      include 'options.inc'
      include 'data.inc'
      double precision g,z
      double precision R,mu,alpha_in,alpha_out,phi_out,TauR,theta
      double precision tmp1,raf1,raf2,tau_out,rkd,kdg,kd,u,S
      integer i,j,band,quad,roc

c     Tout d'abord il faut dterminer si la diffusion concerne le gaz
c     ou un nuage de particules

      call rand_uniforme(R)
      call find_kd_gaz(band,quad,j,z,kdg)
      call find_kd(band,quad,j,z,kd)
      if (kd.ne.0.0D+0) then
         rkd=kdg/kd
      else
         rkd=1.0D+0
      endif

      if (R.le.rkd) then
         roc=1 ! diffusion de Rayleigh
      else
         roc=2 ! diffusion par un nuage de particules
      endif
      
      if (roc.eq.1) then ! diffusion de Rayleigh (gaz)
c     Gnration d'un mu=cos(theta) suivant la fonction de phase
c     de Rayleigh phi(mu)=3/8*(1+mu^2) -- normalise pour
c     avoir int(phi(mu),mu=-1,1)=1
c     Dcoule de la rsolution d'une quation polynomiale du 3me degr
         call rand_uniforme(R)
         u=4.0D+0*R-2.0D+0
         S=(u+dsqrt(1.0D+0+u**2))**(1.0D+0/3.0D+0)
         mu=S-1.0D+0/S
         
      else if (roc.eq.2) then ! diffusion par un nuage de particules

c     Tirage du prochain angle de diffusion (Henyey-Greenstein)
         call find_g(band,quad,j,z,g)
         call rand_uniforme(R)
      
         if (fph.eq.1) then     ! Henyey-Greentein
            if (g.ne.0.D+0) then
               do while (R.eq.(1.D+0-1.D+0/g)/2.D+0)
                  call rand_uniforme(R)
               enddo
            endif
c     Calcul de mu=cos(theta) optimis pour ne pas planter lorsque g=0...
            mu=2.D+0*R*(1.D+0+g)**2*(g*(R-1.D+0)+1.D+0)/
     &           (1.D+0-g*(1.D+0-2.D+0*R))**2-1.D+0
c     Calcul de theta
         else if (fph.eq.2) then ! Inversion de la fonction de phase utilisateur
            do i=2,n_ph(j)
               if ((R.ge.cumul_ph(j,i-1,2)).and.(R.le.cumul_ph(j,i,2)))
     &              then
                  mu=cumul_ph(j,i-1,1)+(cumul_ph(j,i,1)-
     &                 cumul_ph(j,i-1,1))*(R-cumul_ph(j,i-1,2))
     &                 /(cumul_ph(j,i,2)-cumul_ph(j,i-1,2))
               endif
            enddo
         else if (fph.eq.3) then
            if (R.le.g_util) then
               mu=1.0D+0        ! Tir vers l'avant avec une proba g
            else
               mu=dcos(pi*R)    ! Tir isotrope ave une proba 1-g
            endif
         endif

      endif ! sur roc
         
      theta=dacos(mu) ! Dans tous les cas
      
c     Tirage de phi
      call rand_uniforme(R)
      phi_out=2*pi*R
c     Calcul du nouvel angle alpha (/  la verticale)
      tmp1=-dsin(alpha_in)*dcos(phi_out)*dsin(theta)
     &     +dcos(alpha_in)*dcos(theta)
      
      alpha_out=dacos(tmp1)
c     Calcul de la prochaine paisseur optique de diffusion
      call pdf_tau_diff(1,raf1,tau_out,raf2)
      
      return
      end
